local myname="ZifnabsGatherer"
local Me=CreateFrame("Frame",myname)
if not ZifnabsMod then ZifnabsMod={gatherer=Me} else ZifnabsMod.gatherer=Me end
local mod
local L
local C
local db
local current_zone=nil
local fishing=nil
local NODETYPES={[1]="HERB",[2]="ORE",[3]="TREASURE",[4]="FISH",[5]="OTHER"}
local accepttext={[0]="?",[1]="ok",[2]="-"}
local MAXDISTANCE={HERB=0.05,ORE=0.05,TREASURE=0.05,OTHER=0.05,FISH=0.12}
local PLAYERNAME=UnitName("player").." "..GetRealmName()
local widget={}
local config={}

-- Slash Commands
SLASH_ZIFNABSGATHERER1 = "/zg"
SlashCmdList["ZIFNABSGATHERER"]=function (parm) Me:Command(parm) end

-- --------------------------------------------------------------------------------------
-- INITIALIZE
-- --------------------------------------------------------------------------------------
function Me:Initialize(fwinfo)
	mod=fwinfo.mod
	L=fwinfo.L
	C=fwinfo.C
	db=fwinfo.db
	if not db.gatherer then 
		db.gatherer={types={HERB={},ORE={},FISH={},TREASURE={},OTHER={},},
					UNDEFINED={},
					show={},
					mmdistmin={},mmdistmax={},
					active={},
		} 
	end
	if not db.gatherer.active then db.gatherer.active={} end -- Upgrade von r191013a
	if db.gatherer.active[PLAYERNAME]==nil then db.gatherer.active[PLAYERNAME]=true end
	if not db.gatherer.show[PLAYERNAME] then db.gatherer.show[PLAYERNAME]={HERB=true,ORE=true,FISH=true,TREASURE=true,trivial=false,locked=false,OTHER=true} end
	if not db.gatherer.mmdistmin[PLAYERNAME] then db.gatherer.mmdistmin[PLAYERNAME]={HERB=0.5,ORE=0.5,FISH=0.15,TREASURE=0.15,OTHER=0.15} end
	if not db.gatherer.mmdistmax[PLAYERNAME] then db.gatherer.mmdistmax[PLAYERNAME]={HERB=1.5,ORE=1.5,FISH=1.5,TREASURE=1.5,OTHER=1.5} end
	
	BINDING_NAME_ZIFNABS_GATHERER_STORE_CURSOR = L.keybind.gatherer.storecursor
	
	local itemfound={}
	for nodetype,typedata in pairs(db.gatherer.types) do
		for listnr,entry in pairs(typedata.list or {}) do
			local foundhere=0
			for item,count in pairs(entry.list) do 
				itemfound[item]=true 
				foundhere=foundhere+1
			end
			if foundhere==0 then db.gatherer.types[nodetype].list[listnr]=nil end
		end
	end
	
	for nodetype,list in pairs(db.gatherer.UNDEFINED or {}) do
		for item,iinfo in pairs(list) do
			local newid=L.gatherer[nodetype][item]
			if iinfo.accepted==true then iinfo.accepted=1				-- these 2 lines
			elseif iinfo.accepted==false then iinfo.accepted=nil end	-- for upgrade from r191017a
			if newid then 
				local changes=0
				for nr,entry in pairs(db.gatherer.types[nodetype].list) do
					if entry.icon==iinfo.id then
						changes=changes+1
						entry.icon=newid
					end
				end
				mod.f:PrintR(L.msg.itemidchanged,{item=item,old=iinfo.id,new=newid,changes=changes})
				db.gatherer.UNDEFINED[nodetype][item]=nil
			elseif not iinfo.accepted==1 then
				if itemfound[item] then mod.f:PrintR(L.msg.unknownitem,{nodetype=nodetype,item=item,id=iinfo.id}) 
				else
					db.gatherer.UNDEFINED[nodetype][item]=nil
					mod.f:PrintR(L.msg.removeunknownitem,{nodetype=nodetype,item=item,id=iinfo.id})
				end
			end
		end
	end
	Me:Activate(db.gatherer.active[PLAYERNAME])
end

function Me:Activate(set)
	if set==true then
		mod.core:Register4Event("UNIT_SPELLCAST_START","gatherer",Me.SPELLCAST_START)
		mod.core:Register4Event("UNIT_SPELLCAST_CHANNEL_START","gatherer",Me.SPELLCAST_CHANNEL_START)
		mod.core:Register4Event("UNIT_SPELLCAST_CHANNEL_STOP","gatherer",Me.SPELLCAST_CHANNEL_STOP)
		mod.core:Register4Event("ONUPDATE","gatherer",Me.OnUpdate)
		mod.loc:Register4Zone("gatherer",Me.ZoneChanged)
		mod.loc:Register4Map("gatherer",Me.ZoneChanged)
	else
		mod.core:DeRegister4Event("UNIT_SPELLCAST_START","gatherer",Me.SPELLCAST_START)
		mod.core:DeRegister4Event("UNIT_SPELLCAST_CHANNEL_START","gatherer",Me.SPELLCAST_CHANNEL_START)
		mod.core:DeRegister4Event("UNIT_SPELLCAST_CHANNEL_STOP","gatherer",Me.SPELLCAST_CHANNEL_STOP)
		mod.core:DeRegister4Event("ONUPDATE","gatherer",Me.OnUpdate)
		mod.loc:DeRegister4Zone("gatherer",Me.ZoneChanged)
		mod.loc:DeRegister4Map("gatherer",Me.ZoneChanged)
	end
end

-- --------------------------------------------------------------------------------------
-- ZONE CHANGED - generate icon list for Locator
-- --------------------------------------------------------------------------------------
function Me:ZoneChanged(zone,zone_nr)
	if not zone or not zone_nr then _,_,zone,pos,zone_nr=mod.loc:GetLoc() end
	local nodes={}
	for nodetype,dummy in pairs({HERB=1,ORE=2,FISH=3,TREASURE=4,OTHER=5}) do
		if db.gatherer.show[PLAYERNAME][nodetype] and db.gatherer.types[nodetype].list then
			for nr,entry in pairs(db.gatherer.types[nodetype].list) do
				local ex,ey,ezone,eznr=mod.loc:ParseLoc(entry.pos)
				if eznr==zone_nr and ex+ey>0 then
					local text=""
					local trivial=true
					local locked=true
					local hidden=true
					for item,count in pairs(entry.list) do
						text=text.."\n"..item
						local unknown=db.gatherer.UNDEFINED[nodetype][item]
						if not unknown or not unknown.accepted or unknown.accepted~=2 then hidden=false end
						if not trivial or nodetype~="TREASURE" or not L.gatherer.TREASURE[item] or not C.gatherer[L.gatherer.TREASURE[item]] then trivial=false
						elseif not C.gatherer[L.gatherer.TREASURE[item]].crate then trivial=false end
						if not locked or nodetype~="TREASURE" or not L.gatherer.TREASURE[item] or not C.gatherer[L.gatherer.TREASURE[item]] then locked=false
						elseif not C.gatherer[L.gatherer.TREASURE[item]].locked then locked=false end
					end
					local icon=C.gatherer[9999].icon -- UNKNOWN
					if C.gatherer[entry.icon] then icon=C.gatherer[entry.icon].icon end
					if not hidden and (db.gatherer.show[PLAYERNAME].trivial or not trivial) and (db.gatherer.show[PLAYERNAME].locked or not locked) then
						nodes[1+#nodes]={pos=entry.pos,text=string.sub(text,2),mindist=db.gatherer.mmdistmin[PLAYERNAME][nodetype],maxdist=db.gatherer.mmdistmax[PLAYERNAME][nodetype],icon=icon,click=Me.NodeMaint,id=nodetype.." "..entry.pos}
					end
				end
			end
		end
	end
	return nodes
end

-- --------------------------------------------------------------------------------------
-- ON UPDATE
-- --------------------------------------------------------------------------------------
function Me:OnUpdate(elapsed)
	if fishing then
		local cursorname=getglobal("GameTooltipTextLeft1"):GetText() or ""
		local cursorname2=getglobal("GameTooltipTextLeft2"):GetText() or ""
		local item,link = GameTooltip:GetItem();
		if not item and cursorname2=="" and not string.find(cursorname,"(B)") and cursorname~=fishing[fishing.n] and cursorname~="" then
			local ignore=false
			for bag=20,23,1 do
				local bagname=GetItemInfo(GetInventoryItemID("player",bag))
				if bagname==cursorname then ignore=true end
			end
			if not ignore then
				fishing.n=fishing.n+1
				fishing[fishing.n]=cursorname
			end
		end
	end
end

-- --------------------------------------------------------------------------------------
-- SPELLCAST_START
-- --------------------------------------------------------------------------------------
function Me:SPELLCAST_START(who,spellid,rank,more) 
	local cursorname=getglobal("GameTooltipTextLeft1"):GetText() or ""
	local spell=Me:Spellname(rank)
	if spell==L.word.MINING then Me:StoreNode("ORE",cursorname)
	elseif spell==L.word.GATHER_HERBS then Me:StoreNode("HERB",cursorname)
	elseif spell==L.word.PICK_LOCK then Me:StoreNode("TREASURE",cursorname)
	elseif spell==L.word.OPEN then Me:StoreNode("TREASURE",cursorname)
	end
end

-- --------------------------------------------------------------------------------------
-- SPELLCAST_CHANNEL_START
-- --------------------------------------------------------------------------------------
function Me:SPELLCAST_CHANNEL_START(who,spellid,rank,more) 
	if Me:Spellname(rank)==L.word.FISHING then fishing={n=0} end
end

-- --------------------------------------------------------------------------------------
-- SPELLCAST_CHANNEL_STOP
-- --------------------------------------------------------------------------------------
function Me:SPELLCAST_CHANNEL_STOP(who,spellid,rank,more)
	if Me:Spellname(rank)==L.word.FISHING then
		local item=nil
		local unknown=nil
		if fishing and fishing.n then
			for n=fishing.n,1,-1 do
				if item==nil and fishing[n]~=L.word.FishingBobber then
					if L.gatherer.FISH[fishing[n]] then item=fishing[n]
					elseif unknown==nil and not mod.outfitter:ExistsInBags(fishing[n]) and not mod.targets:Seen(fishing[n]) then unknown=fishing[n] end
				end
			end
		end
		if item==nil then item=unknown end
		if item then Me:StoreNode("FISH",item) end
		fishing=nil
	end
end

-- --------------------------------------------------------------------------------------
-- SPELLNAME
-- --------------------------------------------------------------------------------------
function Me:Spellname(rank)
	local spell,srank,icon,casttime,minrange,maxrange=GetSpellInfo(rank)
	return spell
end

-- --------------------------------------------------------------------------------------
-- IS FISHING
-- --------------------------------------------------------------------------------------
function Me:IsFishing()
	if fishing then return true else return nil end
end

-- --------------------------------------------------------------------------------------
-- COMMAND
-- --------------------------------------------------------------------------------------
function Me:Command(cmd)
	local split=mod.f:Split(cmd," ",2)
	local cmd=string.lower(mod.f:Trim(split[1]))
	if cmd=="store" then
		local storeitem=split[2]
		local storetype="OTHER"
		if string.lower(storeitem)=="cursor" then storeitem=getglobal("GameTooltipTextLeft1"):GetText() end
		for gathertype,list in pairs(L.gatherer) do
			for item,info in pairs(list) do
				if string.lower(item)==string.lower(storeitem) then 
					storetype=gathertype 
					storeitem=item
				end
			end
		end
		if storetype=="OTHER" and db.gatherer.UNDEFINED then
			for gathertype,list in pairs(db.gatherer.UNDEFINED) do
				if list[item] then storetype=gathertype end
			end
		end
		Me:StoreNode(storetype,storeitem,true)
	elseif cmd=="show" then
		split=mod.f:Split(cmd.." "..(split[2] or "all")," ",3)
		if db.gatherer.types[string.upper(split[2] or "nil")] then
			db.gatherer.show[PLAYERNAME][string.upper(split[2])]=((split[3] or "+")=="+")
			mod.loc:SetZoneList("gatherer",Me:ZoneChanged())
		else
			for nodetype,truefalse in pairs(db.gatherer.show[PLAYERNAME]) do
				if truefalse then mod.f:Print(nodetype..": +") else mod.f:Print(nodetype..": -") end
			end
		end
	else 
		mod.f:PrintR(L.msg.unknowncmd,{cmd=cmd})
	end
	
end

-- --------------------------------------------------------------------------------------
-- STORE NODE
-- --------------------------------------------------------------------------------------
function Me:StoreNode(nodetype,item,tellifknown)
	local itemid=nil
	if nodetype~="OTHER" then itemid=L.gatherer[nodetype][item] end
	if not itemid then
		if not db.gatherer.UNDEFINED then db.gatherer.UNDEFINED={} end
		if not db.gatherer.UNDEFINED[nodetype] then db.gatherer.UNDEFINED[nodetype]={} end
		if not db.gatherer.UNDEFINED[nodetype][item] then db.gatherer.UNDEFINED[nodetype][item]={} end
		itemid=db.gatherer.UNDEFINED[nodetype][item].id
		if not itemid then
			itemid=db.gatherer.nextundefined or 10000
			db.gatherer.nextundefined=itemid+1
			db.gatherer.UNDEFINED[nodetype][item].id=itemid
			local buzzer="buzzer.mp3"
			if nodetype=="TREASURE" then buzzer=nil end
			mod.f:PrintR(L.msg.unknownitem,{nodetype=nodetype,item=item,id=itemid},buzzer)
		end
	end
	local maxdistance=MAXDISTANCE[nodetype]
	local x,y,zone,pos,zone_nr=mod.loc:GetLoc()
	if zone_nr==0 then mod.f:PrintR(L.msg.nostoreunknownzone,{nodetype=nodetype,item=item,zone=zone},"buzzer.mp3")
	else		
		local nearest={nr=0,distance=99999}
		if not db.gatherer.types[nodetype].nr then db.gatherer.types[nodetype]={nr=0,list={}} end
		for nr,node in pairs(db.gatherer.types[nodetype].list) do
			local distance=mod.loc:Distance(pos,node.pos)
			if distance<nearest.distance then nearest={nr=nr,distance=distance} end
		end
		local rebuildlist=true
		if nearest.distance<maxdistance then
			local entry=db.gatherer.types[nodetype].list[nearest.nr]
			if not entry.list[item] then mod.f:PrintR(L.msg.add2node,{item=item}) 
			elseif tellifknown then mod.f:PrintR(L.msg.alreadyknown,{item=item}) end
			entry.list[item]=1+(entry.list[item] or 0)
			local sum={all=0,highest=0,highnr=0}
			for i,count in pairs(entry.list) do
				sum.all=sum.all+count
				if count>sum.highest then
					sum.highest=count
					sum.highnr=i
				end
			end
			local highid=L.gatherer[nodetype][sum.highnr] or db.gatherer.UNDEFINED[nodetype][sum.highnr].id
			if entry.icon==highid then rebuildlist=false end
			entry.icon=highid
			-- TODO adjust entry.pos with pos with 1/sum.all
		else
			local nr=1+db.gatherer.types[nodetype].nr
			db.gatherer.types[nodetype].nr=nr
			db.gatherer.types[nodetype].list[nr]={pos=pos,icon=itemid,list={[item]=1}}
			mod.f:PrintR(L.msg.newnode,{item=item})
		end
		if rebuildlist then mod.loc:SetZoneList("gatherer",Me:ZoneChanged(zone,zone_nr)) end
	end
end

-- --------------------------------------------------------------------------------------
-- MAINTAIN NODE
-- --------------------------------------------------------------------------------------
function Me:NodeMaint(nodewidget,parse)
	if not widget.maint then
		widget.maint=mod.ui:newFrame(myname.."_NodeMaint",330,100,UIParent,"CENTER","CENTER",0,0,{})
		widget.maintclose=mod.ui:newButton(myname.."_NodeMaint_Close",20,16,widget.maint,"TOPRIGHT","TOPRIGHT",-4,-4,{text="X",click=Me.NodeMaintClose})
		widget.title=mod.ui:addText(myname.."_NodeMaint_Title",330,10,widget.maint,"TOP","TOP",0,-4,11,"CENTER")
		widget.row={}
		for nr=1,10,1 do 
			widget.row[nr]={
				accept=mod.ui:newButton(myname.."_NodeMaint_"..nr.."_Accept",20,16,widget.maint,"TOPRIGHT","TOPRIGHT",-4,-20*nr,{click=Me.NodeMaintButton,text="ok",tt=L.msg.nodeaccept}),
				text=mod.ui:addText(myname.."_NodeMaint_"..nr.."_Text",280,16,widget.maint,"TOPLEFT","TOPLEFT",25,-20*nr,11,"LEFT"),
				delete=mod.ui:newButton(myname.."_NodeMaint_"..nr.."_Delete",20,16,widget.maint,"TOPLEFT","TOPLEFT",5,-20*nr,{click=Me.NodeMaintButton,text="x",tt=L.msg.nodedelete}),
			}
		end
	end
	widget.maint.nodewidget=nodewidget
	local id=mod.f:Split(nodewidget.id," ")
	local x,y,zone=mod.loc:ParseLoc(tonumber(id[2]))
	widget.title:SetText(L.word[id[1]].." : "..x.."/"..y.." "..zone)
	local nr=1
	for listnr,entry in pairs(db.gatherer.types[id[1]].list) do
		if entry.pos==tonumber(id[2]) then
			for item,count in pairs(entry.list) do
				if not db.gatherer.UNDEFINED[id[1]][item] then widget.row[nr].accept:Hide()
				else
					widget.row[nr].accept:SetText(accepttext[db.gatherer.UNDEFINED[id[1]][item].accepted or 0])
					widget.row[nr].accept.tooltipText=L.msg.acceptfinding[db.gatherer.UNDEFINED[id[1]][item].accepted or 0]
					widget.row[nr].accept:Show()
					widget.row[nr].accept.idinfo=id[1].." "..item
				end
				widget.row[nr].text:SetText(count.."x "..item)
				widget.row[nr].delete:Show()
				widget.row[nr].delete.idinfo=id[1].." "..id[2].." "..item
				nr=nr+1
			end
		end
	end
	for h=nr,10,1 do
		widget.row[h].accept:Hide()
		widget.row[h].text:SetText("")
		widget.row[h].delete:Hide()
	end
	widget.maint:Show()
end
function Me:NodeMaintClose() 
	widget.maint:Hide() 
end
function Me:NodeMaintButton(button,parse)
	local nr=tonumber(parse[#parse-1])
	if parse[#parse]=="Accept" then 
		local id=mod.f:Split(button.idinfo," ",2)
		local status=(db.gatherer.UNDEFINED[id[1]][id[2]].accepted or 0)+1
		if status>2 then status=nil end
		db.gatherer.UNDEFINED[id[1]][id[2]].accepted=status
		button:SetText(accepttext[status or 0])
		button.tooltipText=L.msg.acceptfinding[status or 0]
	elseif parse[#parse]=="Delete" then
		local id=mod.f:Split(button.idinfo," ",3)
		Me:DeleteUnknownFinding(id[1],id[3],tonumber(id[2]))
	end
	Me:NodeMaint(widget.maint.nodewidget)
	mod.loc:SetZoneList("gatherer",Me:ZoneChanged())
end
function Me:DeleteUnknownFinding(nodetype,itemname,pos)
	for listnr,entry in pairs(db.gatherer.types[nodetype].list) do
		if not pos or pos==entry.pos then
			entry.list[itemname]=nil
			local remain=0
			for item,count in pairs(entry.list) do remain=remain+1 end
			if remain==0 then tremove(db.gatherer.types[nodetype].list,listnr) end
		end
	end
end
-- --------------------------------------------------------------------------------------
-- IDENTIFY CONFIG FRAMES
-- --------------------------------------------------------------------------------------
function Me:IdentifyConfigs() return {[1]=L.config.gatherer} end

-- --------------------------------------------------------------------------------------
-- START CONFIG
-- --------------------------------------------------------------------------------------
function Me:StartConfig(frame)
	config={utop=0,unknown={},hidden={},all={},subset="all"}
	for nodetype,list in pairs(db.gatherer.UNDEFINED or {}) do	
		for item,iinfo in pairs(list) do
			local status=0
			if iinfo.accepted then status=iinfo.accepted end
			config.all[1+#config.all]={nodetype=nodetype,item=item,status=status,initstatus=status}
			if status==0 then config.unknown[1+#config.unknown]=config.all[#config.all] 
			elseif status==2 then config.hidden[1+#config.hidden]=config.all[#config.all] end
		end
	end
	
	if not widget.active then widget.active=mod.ui:newCheck(myname.."_Active",20,20,frame,"TOPLEFT","TOPLEFT",10,-10,{text=L.config.gatherer.." "..L.word.active,check=db.gatherer.active[PLAYERNAME],click=Me.ConfigChange}) 
	else widget.active:SetChecked(db.gatherer.active[PLAYERNAME]) end
	if not widget.type then widget.type={} end
	for nr,nodetype in pairs(NODETYPES) do
		if not widget.type[nodetype] then
			local show=mod.ui:newCheck(myname.."_Show_"..nodetype,20,20,frame,"TOPLEFT","TOPLEFT",10,-50*nr,{text=L.word[nodetype],check=db.gatherer.show[PLAYERNAME][nodetype],click=Me.ConfigChange})
			local min=mod.ui:newSlider(myname.."_MinDist"..nodetype,200,20,frame,"TOPLEFT","TOPLEFT",120,-50*nr,{div=0.01,min=0,max=500,text="min",val=100*db.gatherer.mmdistmin[PLAYERNAME][nodetype],click=Me.ConfigChange})
			local max=mod.ui:newSlider(myname.."_MaxDist"..nodetype,200,20,frame,"TOPLEFT","TOPLEFT",340,-50*nr,{div=0.01,min=0,max=500,text="max",val=100*db.gatherer.mmdistmax[PLAYERNAME][nodetype],click=Me.ConfigChange})
			widget.type[nodetype]={show=show,min=min,max=max}
			if nodetype=="TREASURE" then 
				widget.trivial=mod.ui:newCheck(myname.."_Trivial",20,20,frame,"TOPLEFT","TOPLEFT",25,-13-50*nr,{text=L.word.trivial,click=Me.ConfigChange}) 
				widget.locked=mod.ui:newCheck(myname.."_Locked",20,20,frame,"TOPLEFT","TOPLEFT",25,-25-50*nr,{text=L.word.locked,click=Me.ConfigChange}) 
			end	
		else
			widget.type[nodetype].show:SetChecked(db.gatherer.show[PLAYERNAME][nodetype])
			widget.type[nodetype].min:SetValue(100*db.gatherer.mmdistmin[PLAYERNAME][nodetype])
			widget.type[nodetype].max:SetValue(100*db.gatherer.mmdistmax[PLAYERNAME][nodetype])
		end
	end
	widget.trivial:SetChecked(db.gatherer.show[PLAYERNAME].trivial)
	widget.locked:SetChecked(db.gatherer.show[PLAYERNAME].locked)
	if not widget.unknown then 
		widget.unknown={}
		for nr=1,5,1 do
			widget.unknown[nr]=	{button=mod.ui:newButton(myname.."_Unknown_"..nr,24,20,frame,"TOPLEFT","TOPLEFT",10,-350-20*nr,{click=Me.ConfigChange}),
								 label=mod.ui:addText(myname.."_Unknown_Text_"..nr,300,20,frame,"TOPLEFT","TOPLEFT",40,-350-20*nr,12,"LEFT"),
								}
		end
		widget.unknown.up=mod.ui:newButton(myname.."_Unknown_Up",20,20,frame,"TOPRIGHT","TOPRIGHT",-4,-370,{click=Me.ShowUnknown,text="^"})
		widget.unknown.down=mod.ui:newButton(myname.."_Unknown_Down",20,20,frame,"TOPRIGHT","TOPRIGHT",-4,-450,{click=Me.ShowUnknown,text="v"})
		widget.unknown.sub=mod.ui:newButton(myname.."_Unknown_Subset",80,20,frame,"TOPRIGHT","TOPRIGHT",-4,-350,{click=Me.ShowUnknownSubset,text=L.word.all,tt=L.msg["gsubset_all"]})
		widget.unknown.help=mod.ui:addText(myname.."_Unknown_Help",580,60,frame,"TOPLEFT","TOPLEFT",10,-300,10,"LEFT","TOP",{text=L.msg.gatheruhelp})
	end
	Me:ShowUnknown(nil,nil)
end

-- --------------------------------------------------------------------------------------
-- SHOW UNKNOWN (incl. Scroll Buttons)
-- --------------------------------------------------------------------------------------
function Me:ShowUnknownSubset(button,parse)
	if config.subset=="all" then config.subset="unknown" 
	elseif config.subset=="unknown" then config.subset="hidden"
	else config.subset="all" end
	widget.unknown.sub:SetText(L.word[config.subset])
	widget.unknown.sub.tooltipText=L.msg["gsubset_"..config.subset]
	config.utop=0
	Me:ShowUnknown(nil,nil)
end

function Me:ShowUnknown(buttonobject,parse)
	if parse and parse[#parse]=="Up" then 
		config.utop=math.max(0,config.utop-1)
	elseif parse and parse[#parse]=="Down" then 
		config.utop=math.max(0,math.min(#config[config.subset]-4,config.utop+1))
	end
	if config.utop==0 then widget.unknown.up:Hide() else widget.unknown.up:Show() end
	if config.utop>=#config[config.subset]-4 then widget.unknown.down:Hide() else widget.unknown.down:Show() end
	for nr=1,5,1 do
		local un=config.utop+nr
		if config[config.subset][un] then
			widget.unknown[nr].button.uitem=un
			Me:Show1Unknown(nr,un)
			widget.unknown[nr].button:Show()
			widget.unknown[nr].label:SetText(config[config.subset][un].nodetype.." "..config[config.subset][un].item)
		else
			widget.unknown[nr].button:Hide()
			widget.unknown[nr].label:SetText("")
		end
	end
end

-- --------------------------------------------------------------------------------------
-- SHOW 1 UNKNOWN
-- --------------------------------------------------------------------------------------
function Me:Show1Unknown(nr,un)
	if config[config.subset][un].status==-1 then 
		widget.unknown[nr].button:SetText("X")
		widget.unknown[nr].button.tooltipText=L.msg.willbedeleted
	elseif config[config.subset][un].status==1 then 
		widget.unknown[nr].button:SetText("ok")
		widget.unknown[nr].button.tooltipText=L.msg.willbeaccepted
	elseif config[config.subset][un].status==2 then 
		widget.unknown[nr].button:SetText("-")
		widget.unknown[nr].button.tooltipText=L.msg.willbehidden
	else
		widget.unknown[nr].button:SetText("?")
		widget.unknown[nr].button.tooltipText=L.msg.willstayunknown
	end
end

-- --------------------------------------------------------------------------------------
-- CONFIG CHANGE
-- --------------------------------------------------------------------------------------
function Me:ConfigChange(button,parse)
	if button.uitem then
		config[config.subset][button.uitem].status=config[config.subset][button.uitem].status+1
		if config[config.subset][button.uitem].status>2 then config[config.subset][button.uitem].status=-1 end
		Me:Show1Unknown(tonumber(parse[#parse]),button.uitem)
	end
	local changed=(widget.active:GetChecked()~=db.gatherer.active[PLAYERNAME])
	changed=changed or (widget.trivial:GetChecked()~=db.gatherer.show[PLAYERNAME].trivial)
	changed=changed or (widget.locked:GetChecked()~=db.gatherer.show[PLAYERNAME].locked)
	for nr,nodetype in pairs(NODETYPES) do
		changed=changed or (widget.type[nodetype].show:GetChecked()~=db.gatherer.show[PLAYERNAME][nodetype])
		changed=changed or (widget.type[nodetype].min:GetValue()~=100*db.gatherer.mmdistmin[PLAYERNAME][nodetype])
		changed=changed or (widget.type[nodetype].max:GetValue()~=100*db.gatherer.mmdistmax[PLAYERNAME][nodetype])
	end
	for un=1,#config.all,1 do
		changed=changed or (config.all[un].status~=config.all[un].initstatus)
	end
	mod.core:SetUnsavedChanges(changed)
end

-- --------------------------------------------------------------------------------------
-- SAVE CONFIG
-- --------------------------------------------------------------------------------------
function Me:SaveConfig(frame)
	db.gatherer.active[PLAYERNAME]=widget.active:GetChecked()
	for nr,nodetype in pairs(NODETYPES) do
		db.gatherer.show[PLAYERNAME][nodetype]=widget.type[nodetype].show:GetChecked()
		db.gatherer.mmdistmin[PLAYERNAME][nodetype]=widget.type[nodetype].min:GetValue()*0.01
		db.gatherer.mmdistmax[PLAYERNAME][nodetype]=widget.type[nodetype].max:GetValue()*0.01
	end
	db.gatherer.show[PLAYERNAME].trivial=widget.trivial:GetChecked()
	db.gatherer.show[PLAYERNAME].locked=widget.locked:GetChecked()
	for nr=1,#config.all,1 do
		if config.all[nr].status~=config.all[nr].initstatus then
			if config.all[nr].status==-1 then 
				db.gatherer.UNDEFINED[config.all[nr].nodetype][config.all[nr].item]=nil
				Me:DeleteUnknownFinding(config.all[nr].nodetype,config.all[nr].item)
			elseif config.all[nr].status==0 then db.gatherer.UNDEFINED[config.all[nr].nodetype][config.all[nr].item].accepted=nil
			else db.gatherer.UNDEFINED[config.all[nr].nodetype][config.all[nr].item].accepted=config.all[nr].status
			end
		end
	end
	Me:Activate(db.gatherer.active[PLAYERNAME])
	Me:StartConfig(frame)
	mod.loc:SetZoneList("gatherer",Me:ZoneChanged())
	return true
end

-- --------------------------------------------------------------------------------------
-- IGNORE CONFIG
-- --------------------------------------------------------------------------------------
function Me:IgnoreConfig(frame) 
	Me:StartConfig(frame)
	return true 
end